#ifndef __FIFO_QUEUE_HH__
#define __FIFO_QUEUE_HH__

/**
 * Well, it is simply a FIFO byte queue. Nothing bind it
 * to audio data, though it's the primary use.
 * (c) 2000 David Le Corfec, under GPL
 */

#include <glib.h>

struct FifoQueue;
typedef struct FifoQueue FifoQueue;

/**
 * Create a queue
 *
 * bytes : buffer size in bytes.
 */
FifoQueue *fifo_new (int bytes);


/**
 * Cleanup.
 */
void fifo_destroy (FifoQueue *fifo);


/*
 * Reset.
 */
void fifo_reset (FifoQueue *fifo);

/*
 * Get maxsize to read.
 */
int fifo_get_size (FifoQueue *fifo);

/*
 * Get max buffer size.
 */
int fifo_get_maxbytes (FifoQueue *fifo);

/**
 * get user data.
 */
gpointer fifo_get_udata (FifoQueue *fifo);

/**
 * set user data.
 */
void fifo_set_udata (FifoQueue *fifo, gpointer data);


/**
 * Read oldest data from a queue.
 *
 * fifo            : queue to read from.
 * buf             : place to store read datas.
 * nbytes          : size to read (in bytes).
 *
 * return: number of bytes read, or -1 if buffer invalid.
 */
int fifo_read (FifoQueue *fifo, char *buf, int nbytes);

/**
 * Append datas to a queue.
 *
 * fifo   : queue to write into.
 * buf    : datas to write.
 * nbytes : number of bytes to write.
 *
 * return : number of bytes written.
 */
int fifo_write (FifoQueue *fifo, const char *buf, int nbytes);


/*
 * debugging. dump n bytes starting from index a.
 */
void fifo_dump (struct FifoQueue *fifo, int a, int n);

#endif
